@@ -11,7 +11,6 @@
 #    under the License.
 
 
-from oslo_log import versionutils
 from oslo_policy import policy
 
 from placement.policies import base
@@ -22,19 +21,6 @@ LIST = PREFIX % 'list'
 UPDATE = PREFIX % 'update'
 BASE_PATH = '/resource_providers/{uuid}/aggregates'
 
-DEPRECATED_REASON = """
-The aggregates API now supports a read-only role by default.
-"""
-
-deprecated_list_aggregates = policy.DeprecatedRule(
-    name=LIST,
-    check_str=base.RULE_ADMIN_API
-)
-deprecated_update_aggregates = policy.DeprecatedRule(
-    name=UPDATE,
-    check_str=base.RULE_ADMIN_API
-)
-
 rules = [
     policy.DocumentedRuleDefault(
         LIST,
@@ -47,9 +33,6 @@ rules = [
             }
         ],
         scope_types=['system'],
-        deprecated_rule=deprecated_list_aggregates,
-        deprecated_reason=DEPRECATED_REASON,
-        deprecated_since=versionutils.deprecated.WALLABY
     ),
     policy.DocumentedRuleDefault(
         UPDATE,
@@ -62,9 +45,6 @@ rules = [
             }
         ],
         scope_types=['system'],
-        deprecated_rule=deprecated_update_aggregates,
-        deprecated_reason=DEPRECATED_REASON,
-        deprecated_since=versionutils.deprecated.WALLABY
     ),
 ]
 
