@@ -10,13 +10,15 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import mock
+from unittest import mock
+
 import os_resource_classes as orc
 from oslo_utils.fixture import uuidsentinel
 
 from placement import exception
 from placement.objects import allocation as alloc_obj
 from placement.objects import consumer as consumer_obj
+from placement.objects import consumer_type as ct_obj
 from placement.objects import inventory as inv_obj
 from placement.objects import usage as usage_obj
 from placement.tests.functional.db import test_base as tb
@@ -92,10 +94,16 @@ class TestAllocation(tb.PlacementDbBaseTestCase):
                              step_size=64,
                              allocation_ratio=1.5)
 
+        # Create an INSTANCE consumer type
+        ct = ct_obj.ConsumerType(self.ctx, name='INSTANCE')
+        ct.create()
+        # Save consumer type id for later confirmation.
+        ct_id = ct.id
+
         # Create a consumer representing the instance
         inst_consumer = consumer_obj.Consumer(
             self.ctx, uuid=uuidsentinel.instance, user=self.user_obj,
-            project=self.project_obj)
+            project=self.project_obj, consumer_type_id=ct_id)
         inst_consumer.create()
 
         # Now create an allocation that represents a move operation where the
@@ -174,6 +182,10 @@ class TestAllocation(tb.PlacementDbBaseTestCase):
 
         self.assertEqual(2, len(consumer_allocs))
 
+        # check the allocations have the expected INSTANCE consumer type
+        self.assertEqual(ct_id, consumer_allocs[0].consumer.consumer_type_id)
+        self.assertEqual(ct_id, consumer_allocs[1].consumer.consumer_type_id)
+
     def test_get_all_by_resource_provider(self):
         rp, allocation = self._make_allocation(tb.DISK_INVENTORY,
                                                tb.DISK_ALLOCATION)
@@ -624,40 +636,42 @@ class TestAllocationListCreateDelete(tb.PlacementDbBaseTestCase):
         ]
 
         # Make sure the right exception happens when the retry loop expires.
-        with mock.patch.object(alloc_obj, 'RP_CONFLICT_RETRY_COUNT', 0):
-            self.assertRaises(
-                exception.ResourceProviderConcurrentUpdateDetected,
-                alloc_obj.replace_all, self.ctx, alloc_list)
-            mock_log.warning.assert_called_with(
-                'Exceeded retry limit of %d on allocations write', 0)
+        self.conf_fixture.config(allocation_conflict_retry_count=0,
+                                 group='placement')
+        self.assertRaises(
+            exception.ResourceProviderConcurrentUpdateDetected,
+            alloc_obj.replace_all, self.ctx, alloc_list)
+        mock_log.warning.assert_called_with(
+            'Exceeded retry limit of %d on allocations write', 0)
 
         # Make sure the right thing happens after a small number of failures.
         # There's a bit of mock magic going on here to enusre that we can
         # both do some side effects on _set_allocations as well as have the
         # real behavior. Two generation conflicts and then a success.
         mock_log.reset_mock()
-        with mock.patch.object(alloc_obj, 'RP_CONFLICT_RETRY_COUNT', 3):
-            unmocked_set = alloc_obj._set_allocations
-            with mock.patch('placement.objects.allocation.'
-                            '_set_allocations') as mock_set:
-                exceptions = iter([
-                    exception.ResourceProviderConcurrentUpdateDetected(),
-                    exception.ResourceProviderConcurrentUpdateDetected(),
-                ])
+        self.conf_fixture.config(allocation_conflict_retry_count=3,
+                                 group='placement')
+        unmocked_set = alloc_obj._set_allocations
+        with mock.patch('placement.objects.allocation.'
+                        '_set_allocations') as mock_set:
+            exceptions = iter([
+                exception.ResourceProviderConcurrentUpdateDetected(),
+                exception.ResourceProviderConcurrentUpdateDetected(),
+            ])
 
-                def side_effect(*args, **kwargs):
-                    try:
-                        raise next(exceptions)
-                    except StopIteration:
-                        return unmocked_set(*args, **kwargs)
+            def side_effect(*args, **kwargs):
+                try:
+                    raise next(exceptions)
+                except StopIteration:
+                    return unmocked_set(*args, **kwargs)
 
-                mock_set.side_effect = side_effect
-                alloc_obj.replace_all(self.ctx, alloc_list)
-                self.assertEqual(2, mock_log.debug.call_count)
-                mock_log.debug.called_with(
-                    'Retrying allocations write on resource provider '
-                    'generation conflict')
-                self.assertEqual(3, mock_set.call_count)
+            mock_set.side_effect = side_effect
+            alloc_obj.replace_all(self.ctx, alloc_list)
+            self.assertEqual(2, mock_log.debug.call_count)
+            mock_log.debug.called_with(
+                'Retrying allocations write on resource provider '
+                'generation conflict')
+            self.assertEqual(3, mock_set.call_count)
 
         # Confirm we're using a different rp object after the change
         # and that it has a higher generation.
