@@ -99,12 +99,21 @@ class CreateIncompleteAllocationsMixin(object):
     """Mixin for test setup to create some allocations with missing consumers
     """
 
+    @db_api.placement_context_manager.writer
+    def _create_leftover_consumer(self, ctx):
+        ins_stmt = CONSUMER_TBL.insert().values(
+            uuid=uuids.unknown_consumer,
+            project_id=999,
+            user_id=999)
+        ctx.session.execute(ins_stmt)
+
     @db_api.placement_context_manager.writer
     def _create_incomplete_allocations(self, ctx, num_of_consumer_allocs=1):
         # Create some allocations with consumers that don't exist in the
         # consumers table to represent old allocations that we expect to be
         # "cleaned up" with consumers table records that point to the sentinel
         # project/user records.
+        self._create_leftover_consumer(ctx)
         c1_missing_uuid = uuids.c1_missing
         c2_missing_uuid = uuids.c2_missing
         c3_missing_uuid = uuids.c3_missing
@@ -141,43 +150,6 @@ class CreateIncompleteConsumersTestCase(
         super(CreateIncompleteConsumersTestCase, self).setUp()
         self.ctx = self.context
 
-    @db_api.placement_context_manager.reader
-    def _check_incomplete_consumers(self, ctx):
-        config = ctx.config
-        incomplete_project_id = config.placement.incomplete_consumer_project_id
-
-        # Verify we have a record in projects for the missing sentinel
-        sel = PROJECT_TBL.select(
-            PROJECT_TBL.c.external_id == incomplete_project_id)
-        rec = ctx.session.execute(sel).first()
-        self.assertEqual(incomplete_project_id, rec['external_id'])
-        incomplete_proj_id = rec['id']
-
-        # Verify we have a record in users for the missing sentinel
-        incomplete_user_id = config.placement.incomplete_consumer_user_id
-        sel = user_obj.USER_TBL.select(
-            USER_TBL.c.external_id == incomplete_user_id)
-        rec = ctx.session.execute(sel).first()
-        self.assertEqual(incomplete_user_id, rec['external_id'])
-        incomplete_user_id = rec['id']
-
-        # Verify there are records in the consumers table for our old
-        # allocation records created in the pre-migration setup and that the
-        # projects and users referenced in those consumer records point to the
-        # incomplete project/user
-        sel = CONSUMER_TBL.select(CONSUMER_TBL.c.uuid == uuids.c1_missing)
-        missing_c1 = ctx.session.execute(sel).first()
-        self.assertEqual(incomplete_proj_id, missing_c1['project_id'])
-        self.assertEqual(incomplete_user_id, missing_c1['user_id'])
-        sel = CONSUMER_TBL.select(CONSUMER_TBL.c.uuid == uuids.c2_missing)
-        missing_c2 = ctx.session.execute(sel).first()
-        self.assertEqual(incomplete_proj_id, missing_c2['project_id'])
-        self.assertEqual(incomplete_user_id, missing_c2['user_id'])
-
-        # Ensure there are no more allocations with incomplete consumers
-        res = _get_allocs_with_no_consumer_relationship(ctx)
-        self.assertEqual(0, len(res))
-
     def test_create_incomplete_consumers(self):
         """Test the online data migration that creates incomplete consumer
         records along with the incomplete consumer project/user records.
