@@ -14,18 +14,17 @@
 
 
 import datetime
+from unittest import mock
 
 import fixtures
 import microversion_parse
-import mock
 from oslo_middleware import request_id
 from oslo_utils.fixture import uuidsentinel
 from oslo_utils import timeutils
 import testtools
 import webob
 
-import six
-
+import placement
 from placement import context
 from placement import lib as pl
 from placement import microversion
@@ -163,7 +162,7 @@ class QueryParamsSchemaTestCase(testtools.TestCase):
         error = self.assertRaises(webob.exc.HTTPBadRequest,
                                   util.validate_query_params,
                                   req, schema)
-        self.assertIn('Invalid query string parameters', six.text_type(error))
+        self.assertIn('Invalid query string parameters', str(error))
 
 
 class TestJSONErrorFormatter(testtools.TestCase):
@@ -396,14 +395,16 @@ class TestNormalizeResourceQsParam(testtools.TestCase):
         )
 
 
-class TestNormalizeTraitsQsParam(testtools.TestCase):
+class TestNormalizeTraitsQsParamLegacy(testtools.TestCase):
 
     def test_one(self):
         trait = 'HW_CPU_X86_VMX'
         # Various whitespace permutations
         for fmt in ('%s', ' %s', '%s ', ' %s ', '  %s  '):
-            self.assertEqual(set([trait]),
-                             util.normalize_traits_qs_param(fmt % trait))
+            self.assertEqual(
+                set([trait]),
+                util.normalize_traits_qs_param_to_legacy_value(fmt % trait)
+            )
 
     def test_multiple(self):
         traits = (
@@ -415,12 +416,15 @@ class TestNormalizeTraitsQsParam(testtools.TestCase):
         )
         self.assertEqual(
             set(traits),
-            util.normalize_traits_qs_param('%s, %s,%s , %s ,  %s  ' % traits))
+            util.normalize_traits_qs_param_to_legacy_value(
+                '%s, %s,%s , %s ,  %s  ' % traits)
+        )
 
     def test_400_all_empty(self):
         for qs in ('', ' ', '   ', ',', ' , , '):
             self.assertRaises(
-                webob.exc.HTTPBadRequest, util.normalize_traits_qs_param, qs)
+                webob.exc.HTTPBadRequest,
+                util.normalize_traits_qs_param_to_legacy_value, qs)
 
     def test_400_some_empty(self):
         traits = (
@@ -429,8 +433,210 @@ class TestNormalizeTraitsQsParam(testtools.TestCase):
             'STORAGE_DISK_SSD',
         )
         for fmt in ('%s,,%s,%s', ',%s,%s,%s', '%s,%s,%s,', ' %s , %s ,  , %s'):
-            self.assertRaises(webob.exc.HTTPBadRequest,
-                              util.normalize_traits_qs_param, fmt % traits)
+            self.assertRaises(
+                webob.exc.HTTPBadRequest,
+                util.normalize_traits_qs_param_to_legacy_value, fmt % traits)
+
+
+class TestNormalizeTraitsQsParam(testtools.TestCase):
+
+    def test_one(self):
+        trait = 'HW_CPU_X86_VMX'
+        # Various whitespace permutations
+        for fmt in ('%s', ' %s', '%s ', ' %s ', '  %s  '):
+            self.assertEqual(
+                ([{trait}], set()),
+                util.normalize_traits_qs_param(fmt % trait)
+            )
+
+    def test_multiple(self):
+        traits = (
+            'HW_CPU_X86_VMX',
+            'HW_GPU_API_DIRECT3D_V12_0',
+            'HW_NIC_OFFLOAD_RX',
+            'CUSTOM_GOLD',
+            'STORAGE_DISK_SSD',
+        )
+        self.assertEqual(
+            ([{trait} for trait in traits], set()),
+            util.normalize_traits_qs_param(
+                '%s, %s,%s , %s ,  %s  ' % traits)
+        )
+
+    def test_400_all_empty(self):
+        for qs in ('', ' ', '   ', ',', ' , , '):
+            self.assertRaises(
+                webob.exc.HTTPBadRequest,
+                util.normalize_traits_qs_param, qs)
+
+    def test_400_some_empty(self):
+        traits = (
+            'HW_NIC_OFFLOAD_RX',
+            'CUSTOM_GOLD',
+            'STORAGE_DISK_SSD',
+        )
+        for fmt in (
+            '%s,,%s,%s',
+            ',%s,%s,%s',
+            '%s,%s,%s,',
+            ' %s , %s ,  , %s',
+            '!,%s,%s,%s',
+        ):
+            self.assertRaises(
+                webob.exc.HTTPBadRequest,
+                util.normalize_traits_qs_param,
+                fmt % traits,
+                allow_forbidden=True,
+            )
+
+    def test_multiple_with_forbidden(self):
+        traits = (
+            '!HW_CPU_X86_VMX',
+            'HW_GPU_API_DIRECT3D_V12_0',
+            '!HW_NIC_OFFLOAD_RX',
+            'CUSTOM_GOLD',
+            '!STORAGE_DISK_SSD',
+        )
+
+        self.assertRaises(
+            webob.exc.HTTPBadRequest,
+            util.normalize_traits_qs_param,
+            '%s, %s,%s , %s ,  %s  ' % traits,
+            allow_forbidden=False)
+
+        self.assertEqual(
+            (
+                [{'HW_GPU_API_DIRECT3D_V12_0'}, {'CUSTOM_GOLD'}],
+                {'HW_CPU_X86_VMX', 'HW_NIC_OFFLOAD_RX', 'STORAGE_DISK_SSD'}),
+            util.normalize_traits_qs_param(
+                '%s, %s,%s , %s ,  %s  ' % traits, allow_forbidden=True)
+        )
+
+    def test_any_traits(self):
+        param = 'in:T1 ,T2 , T3'
+        self.assertRaises(
+            webob.exc.HTTPBadRequest,
+            util.normalize_traits_qs_param,
+            param,
+            allow_any_traits=False
+        )
+        self.assertEqual(
+            ([{'T1', 'T2', 'T3'}], set()),
+            util.normalize_traits_qs_param(param, allow_any_traits=True)
+        )
+
+    def test_any_traits_not_mix_with_forbidden(self):
+        param = 'in:T1 ,!T2 , T3'
+        self.assertRaises(
+            webob.exc.HTTPBadRequest,
+            util.normalize_traits_qs_param,
+            param,
+            allow_forbidden=True,
+            allow_any_traits=True,
+        )
+
+
+class TestNormalizeTraitsQsParams(testtools.TestCase):
+    @staticmethod
+    def _get_req(qstring, version):
+        req = webob.Request.blank(
+            '?' + qstring,
+        )
+        mv_parsed = microversion_parse.Version(*version)
+        mv_parsed.max_version = microversion_parse.parse_version_string(
+            microversion.max_version_string()
+        )
+        mv_parsed.min_version = microversion_parse.parse_version_string(
+            microversion.min_version_string()
+        )
+        req.environ[placement.microversion.MICROVERSION_ENVIRON] = mv_parsed
+        return req
+
+    def test_suffix(self):
+        req = self._get_req('required=!BAZ&requiredX=FOO,BAR', (1, 38))
+
+        required, forbidden = util.normalize_traits_qs_params(req, suffix='')
+
+        self.assertEqual([], required)
+        self.assertEqual({'BAZ'}, forbidden)
+
+        required, forbidden = util.normalize_traits_qs_params(req, suffix='X')
+
+        self.assertEqual([{'FOO'}, {'BAR'}], required)
+        self.assertEqual(set(), forbidden)
+
+    def test_allow_forbidden_1_21(self):
+        req = self._get_req('required=!BAZ', (1, 21))
+
+        ex = self.assertRaises(
+            webob.exc.HTTPBadRequest,
+            util.normalize_traits_qs_params,
+            req,
+            suffix='',
+        )
+
+        self.assertIn(
+            "Invalid query string parameters: Expected 'required' parameter "
+            "value of the form: HW_CPU_X86_VMX,CUSTOM_MAGIC. Got: !BAZ",
+            str(ex),
+        )
+
+    def test_allow_forbidden_1_22(self):
+        req = self._get_req('required=!BAZ', (1, 22))
+
+        required, forbidden = util.normalize_traits_qs_params(req, suffix='')
+
+        self.assertEqual([], required)
+        self.assertEqual({'BAZ'}, forbidden)
+
+    def test_repeated_param_1_38(self):
+        req = self._get_req('required=FOO,!BAR&required=BAZ', (1, 38))
+
+        required, forbidden = util.normalize_traits_qs_params(req, suffix='')
+
+        self.assertEqual([{'BAZ'}], required)
+        self.assertEqual(set(), forbidden)
+
+    def test_allow_any_traits_1_38(self):
+        req = self._get_req('required=in:FOO,BAZ', (1, 38))
+
+        ex = self.assertRaises(
+            webob.exc.HTTPBadRequest,
+            util.normalize_traits_qs_params,
+            req,
+            suffix='',
+        )
+
+        self.assertIn(
+            "Invalid query string parameters: "
+            "The format 'in:HW_CPU_X86_VMX,CUSTOM_MAGIC' only supported "
+            "since microversion 1.39. Got: in:FOO,BAZ",
+            str(ex),
+        )
+
+    def test_allow_any_traits_1_39(self):
+        req = self._get_req('required=in:FOO,BAZ', (1, 39))
+
+        required, forbidden = util.normalize_traits_qs_params(req, suffix='')
+
+        self.assertEqual([{'FOO', 'BAZ'}], required)
+        self.assertEqual(set(), forbidden)
+
+    def test_repeated_param_1_39(self):
+        req = self._get_req(
+            'required=in:T1,T2'
+            '&required=T3,!T4'
+            '&required=in:T5,T6'
+            '&required=!T7,T8',
+            (1, 39)
+        )
+
+        required, forbidden = util.normalize_traits_qs_params(req, suffix='')
+
+        self.assertEqual(
+            [{'T1', 'T2'}, {'T3'}, {'T5', 'T6'}, {'T8'}],
+            required)
+        self.assertEqual({'T4', 'T7'}, forbidden)
 
 
 class TestParseQsRequestGroups(testtools.TestCase):
@@ -472,10 +678,9 @@ class TestParseQsRequestGroups(testtools.TestCase):
                     'VCPU': 2,
                     'MEMORY_MB': 2048,
                 },
-                required_traits={
-                    'HW_CPU_X86_VMX',
-                    'CUSTOM_GOLD',
-                },
+                required_traits=[
+                    {'HW_CPU_X86_VMX'}, {'CUSTOM_GOLD'}
+                ],
             ),
         ]
         self.assertRequestGroupsEqual(expected, self.do_parse(qs))
@@ -564,18 +769,17 @@ class TestParseQsRequestGroups(testtools.TestCase):
                     'VCPU': 2,
                     'MEMORY_MB': 2048,
                 },
-                required_traits={
-                    'HW_CPU_X86_VMX',
-                    'CUSTOM_GOLD',
-                },
+                required_traits=[
+                    {'HW_CPU_X86_VMX'}, {'CUSTOM_GOLD'}
+                ],
             ),
             pl.RequestGroup(
                 resources={
                     'CUSTOM_MAGIC': 123,
                 },
-                required_traits={
-                    'CUSTOM_GOLD',
-                },
+                required_traits=[
+                    {'CUSTOM_GOLD'}
+                ],
             ),
             pl.RequestGroup(
                 resources={
@@ -602,19 +806,19 @@ class TestParseQsRequestGroups(testtools.TestCase):
                     'MEMORY_MB': 4096,
                     'DISK_GB': 10,
                 },
-                required_traits={
-                    'HW_CPU_X86_VMX',
-                    'CUSTOM_MEM_FLASH',
-                    'STORAGE_DISK_SSD',
-                },
+                required_traits=[
+                    {'HW_CPU_X86_VMX'},
+                    {'CUSTOM_MEM_FLASH'},
+                    {'STORAGE_DISK_SSD'}
+                ],
             ),
             pl.RequestGroup(
                 resources={
                     'SRIOV_NET_VF': 2,
                 },
-                required_traits={
-                    'CUSTOM_PHYSNET_PRIVATE',
-                },
+                required_traits=[
+                    {'CUSTOM_PHYSNET_PRIVATE'},
+                ],
             ),
             pl.RequestGroup(
                 resources={
@@ -622,10 +826,10 @@ class TestParseQsRequestGroups(testtools.TestCase):
                     'NET_INGRESS_BYTES_SEC': 20000,
                     'NET_EGRESS_BYTES_SEC': 10000,
                 },
-                required_traits={
-                    'CUSTOM_SWITCH_BIG',
-                    'CUSTOM_PHYSNET_PROD',
-                },
+                required_traits=[
+                    {'CUSTOM_SWITCH_BIG'},
+                    {'CUSTOM_PHYSNET_PROD'},
+                ],
             ),
             pl.RequestGroup(
                 resources={
@@ -819,9 +1023,9 @@ class TestParseQsRequestGroups(testtools.TestCase):
                     'VCPU': 2,
                     'MEMORY_MB': 2048,
                 },
-                required_traits={
-                    'CUSTOM_PHYSNET1',
-                },
+                required_traits=[
+                    {'CUSTOM_PHYSNET1'},
+                ],
                 forbidden_traits={
                     'CUSTOM_SWITCH_BIG',
                 }
@@ -832,7 +1036,7 @@ class TestParseQsRequestGroups(testtools.TestCase):
             "value of the form: HW_CPU_X86_VMX,CUSTOM_MAGIC. Got: "
             "CUSTOM_PHYSNET1,!CUSTOM_SWITCH_BIG")
         exc = self.assertRaises(webob.exc.HTTPBadRequest, self.do_parse, qs)
-        self.assertEqual(expected_message, six.text_type(exc))
+        self.assertEqual(expected_message, str(exc))
         self.assertRequestGroupsEqual(
             expected_forbidden, self.do_parse(qs, version=(1, 22)))
 
@@ -846,7 +1050,7 @@ class TestParseQsRequestGroups(testtools.TestCase):
 
         exc = self.assertRaises(
             webob.exc.HTTPBadRequest, self.do_parse, qs, version=(1, 22))
-        self.assertEqual(expected_message, six.text_type(exc))
+        self.assertEqual(expected_message, str(exc))
 
     def test_forbidden_two_groups(self):
         qs = ('resources=VCPU:2,MEMORY_MB:2048&resources1=CUSTOM_MAGIC:1'
@@ -863,9 +1067,9 @@ class TestParseQsRequestGroups(testtools.TestCase):
                 resources={
                     'CUSTOM_MAGIC': 1,
                 },
-                required_traits={
-                    'CUSTOM_PHYSNET1',
-                },
+                required_traits=[
+                    {'CUSTOM_PHYSNET1'},
+                ],
                 forbidden_traits={
                     'CUSTOM_PHYSNET2',
                 }
@@ -884,9 +1088,9 @@ class TestParseQsRequestGroups(testtools.TestCase):
                 resources={
                     'CUSTOM_MAGIC': 1,
                 },
-                required_traits={
-                    'CUSTOM_PHYSNET1',
-                }
+                required_traits=[
+                    {'CUSTOM_PHYSNET1'},
+                ],
             ),
             pl.RequestGroup(
                 use_same_provider=True,
@@ -936,9 +1140,9 @@ class TestParseQsRequestGroups(testtools.TestCase):
                 resources={
                     'CUSTOM_MAGIC': 1,
                 },
-                required_traits={
-                    'CUSTOM_PHYSNET1',
-                }
+                required_traits=[
+                    {'CUSTOM_PHYSNET1'},
+                ],
             ),
         ]
         self.assertRequestGroupsEqual(
@@ -946,6 +1150,127 @@ class TestParseQsRequestGroups(testtools.TestCase):
         self.assertRaises(
             webob.exc.HTTPBadRequest, self.do_parse, qs, version=(1, 22))
 
+    def test_any_traits_1_38(self):
+        qs = 'resources1=RABBIT:1&required1=in:WHITE,BLACK'
+
+        exc = self.assertRaises(
+            webob.exc.HTTPBadRequest, self.do_parse, qs, version=(1, 38))
+        self.assertIn(
+            "The format 'in:HW_CPU_X86_VMX,CUSTOM_MAGIC' only supported since "
+            "microversion 1.39.",
+            str(exc))
+
+    def test_any_traits_1_39(self):
+        qs = 'resources1=RABBIT:1&required1=in:WHITE,BLACK'
+        expected = [
+            pl.RequestGroup(
+                use_same_provider=True,
+                resources={
+                    'RABBIT': 1,
+                },
+                required_traits=[
+                    {'WHITE', 'BLACK'},
+                ],
+            ),
+        ]
+
+        self.assertRequestGroupsEqual(
+            expected, self.do_parse(qs, version=(1, 39)))
+
+    def test_any_traits_repeated(self):
+        qs = 'resources1=CUSTOM_MAGIC:1&required1=in:T1,T2&required1=T3,!T4'
+        expected = [
+            pl.RequestGroup(
+                use_same_provider=True,
+                resources={
+                    'CUSTOM_MAGIC': 1,
+                },
+                required_traits=[
+                    {'T1', 'T2'},
+                    {'T3'},
+                ],
+                forbidden_traits={
+                    'T4'
+                },
+            ),
+        ]
+
+        self.assertRequestGroupsEqual(
+            expected, self.do_parse(qs, version=(1, 39)))
+
+    def test_any_traits_multiple_groups(self):
+        qs = ('resources=RABBIT:1&required=in:WHITE,BLACK&'
+              'resources2=CAT:2&required2=in:SILVER,RED&required2=!SPOTTED')
+        expected = [
+            pl.RequestGroup(
+                use_same_provider=False,
+                resources={
+                    'RABBIT': 1,
+                },
+                required_traits=[
+                    {'WHITE', 'BLACK'},
+                ],
+                forbidden_traits={
+                },
+            ),
+            pl.RequestGroup(
+                use_same_provider=True,
+                resources={
+                    'CAT': 2,
+                },
+                required_traits=[
+                    {'SILVER', 'RED'},
+                ],
+                forbidden_traits={
+                    'SPOTTED'
+                },
+            ),
+        ]
+
+        self.assertRequestGroupsEqual(
+            expected, self.do_parse(qs, version=(1, 39)))
+
+    def test_any_traits_forbidden_conflict(self):
+        # going against one part of an OR expression is not a conflict as the
+        # other parts still can match and fulfill the query
+        qs = ('resources=VCPU:2'
+              '&required=in:CUSTOM_PHYSNET1,CUSTOM_PHYSNET2'
+              '&required=!CUSTOM_PHYSNET1')
+
+        rgs = self.do_parse(qs, version=(1, 39))
+        self.assertEqual(1, len(rgs))
+
+        # but going against all parts of an OR expression is a conflict
+        qs = ('resources=VCPU:2'
+              '&required=in:CUSTOM_PHYSNET1,CUSTOM_PHYSNET2'
+              '&required=!CUSTOM_PHYSNET1,!CUSTOM_PHYSNET2')
+
+        expected_message = (
+            'Conflicting required and forbidden traits found '
+            'in the following traits keys: required: '
+            '(CUSTOM_PHYSNET1, CUSTOM_PHYSNET2)')
+
+        exc = self.assertRaises(
+            webob.exc.HTTPBadRequest, self.do_parse, qs, version=(1, 39))
+        self.assertEqual(expected_message, str(exc))
+
+    def test_stringification(self):
+        agg1 = uuidsentinel.agg1
+        agg2 = uuidsentinel.agg2
+        qs = (f'resources1=CAT:2&required1=in:SILVER,RED&'
+              f'required1=TABBY,!SPOTTED&member_of1=in:{agg1},{agg2}')
+
+        rgs = self.do_parse(qs, version=(1, 39))
+        self.assertEqual(1, len(rgs))
+        self.assertEqual(
+            'RequestGroup('
+            'use_same_provider=True, '
+            'resources={CAT:2}, '
+            'traits=((RED or SILVER) and TABBY and !SPOTTED), '
+            f'aggregates=[[{", ".join(sorted([agg1, agg2]))}]])',
+            str(rgs[0])
+        )
+
 
 class TestPickLastModified(base.ContextTestCase):
 
